@@ -12,10 +12,11 @@ from twisted.internet.defer import Deferred, DeferredQueue
 import getpass
 import math
 import operator
+import time
 
 KEYMAP = {
     'bsp': rfb.KEY_BackSpace,
-    'tab':  rfb.KEY_Tab,
+    'tab': rfb.KEY_Tab,
     'return': rfb.KEY_Return,
     'enter': rfb.KEY_Return,
     'esc': rfb.KEY_Escape,
@@ -90,9 +91,18 @@ KEYMAP = {
 }
 
 
+def ImageFactory():
+    """ Wrap importing PIL.Image so vncdotool can be used without
+    PIL being installed.  Of course capture and expect won't work
+    but at least we can still offer key, type, press and move.
+    """
+    from PIL import Image
+    return Image
+
 class VNCDoToolClient(rfb.RFBClient):
     x = 0
     y = 0
+    buttons = 0
     screen = None
 
     def keyPress(self, key):
@@ -120,9 +130,31 @@ class VNCDoToolClient(rfb.RFBClient):
             button: int: [1-n]
 
         """
-        buttons = 1 << (button - 1)
+        buttons = self.buttons | (1 << (button - 1))
         self.pointerEvent(self.x, self.y, buttonmask=buttons)
-        self.pointerEvent(self.x, self.y, buttonmask=0)
+        self.pointerEvent(self.x, self.y, buttonmask=self.buttons)
+
+        return self
+
+    def mouseDown(self, button):
+        """ Send a mouse button down at the last set position
+
+            button: int: [1-n]
+
+        """
+        self.buttons |= 1 << (button - 1)
+        self.pointerEvent(self.x, self.y, buttonmask=self.buttons)
+
+        return self
+
+    def mouseUp(self, button):
+        """ Send mouse button released at the last set position
+
+            button: int: [1-n]
+
+        """
+        self.buttons &= ~(1 << (button - 1))
+        self.pointerEvent(self.x, self.y, buttonmask=self.buttons)
 
         return self
 
@@ -149,7 +181,7 @@ class VNCDoToolClient(rfb.RFBClient):
                     screen and target image
         """
         self.framebufferUpdateRequest()
-        self.expected = self.image().open(filename).histogram()
+        self.expected = ImageFactory().open(filename).histogram()
         self.deferred = Deferred()
         d = self.updates.get()
         d.addCallback(self._expectCompare, maxrms)
@@ -158,46 +190,66 @@ class VNCDoToolClient(rfb.RFBClient):
 
     def _expectCompare(self, image, maxrms):
         hist = image.histogram()
+        if len(hist) == len(self.expected):
+            rms = math.sqrt(
+                        reduce(operator.add, map(lambda a, b: (a - b) ** 2,
+                            hist, self.expected)) / len(hist))
 
-        rms = math.sqrt(
-            reduce(operator.add, map(
-               lambda a, b: (a - b) ** 2, hist, self.expected)) / len(hist))
-
-        self.log('rms %d', rms)
-
-        if rms <= maxrms:
-            self.deferred.callback(self)
-            self.deferred = None
-            return
+            self.log('rms', int(rms))
+            if rms <= maxrms:
+                self.deferred.callback(self)
+                self.deferred = None
+                return
 
         d = self.updates.get()
         d.addCallback(self._expectCompare, maxrms)
+        self.framebufferUpdateRequest(incremental=1)
 
     def mouseMove(self, x, y):
         """ Move the mouse pointer to position (x, y)
         """
         self.x, self.y = x, y
-        self.pointerEvent(x, y)
+        self.pointerEvent(x, y, self.buttons)
+        return self
+
+    def mouseDrag(self, x, y, step=1):
+        """ Move the mouse point to position (x, y) in increments of step
+        """
+        if x < self.x:
+            xsteps = [self.x - i for i in xrange(step, self.x - x + 1, step)]
+        else:
+            xsteps = xrange(self.x, x, step)
+
+        if y < self.y:
+            ysteps = [self.y - i for i in xrange(step, self.y - y + 1, step)]
+        else:
+            ysteps = xrange(self.y, y, step)
+
+        for ypos in ysteps:
+            time.sleep(.2)
+            self.mouseMove(self.x, ypos)
+
+        for xpos in xsteps:
+            time.sleep(.2)
+            self.mouseMove(xpos, self.y)
+
+        self.mouseMove(x, y)
+
         return self
 
     def log(self, fmt, *args):
         if self.factory.logger:
             self.factory.logger(fmt, *args)
 
-    def image(self):
-        """ Wrap importing PIL.Image so vncdotool can be used without
-        PIL being installed.  Of course capture and expect won't work
-        but at least we can still offer key, type, press and move.
-        """
-        from PIL import Image
-        return Image
-
     #
     # base customizations
     #
+    def connectionMade(self):
+        rfb.RFBClient.connectionMade(self)
+        self.updates = DeferredQueue()
+
     def vncConnectionMade(self):
         self.setPixelFormat()
-        self.updates = DeferredQueue()
         self.factory.clientConnectionMade(self)
 
     def bell(self):
@@ -212,7 +264,7 @@ class VNCDoToolClient(rfb.RFBClient):
 
     def updateRectangle(self, x, y, width, height, data):
         size = (width, height)
-        update = self.image().fromstring('RGB', size, data, 'raw', 'RGBX')
+        update = ImageFactory().fromstring('RGB', size, data, 'raw', 'RGBX')
         if not self.screen:
             self.screen = update
         # track screen upward screen resizes, often occur during os boot
